;****************************
;*  C3 Multicart Menu V1.0  *
;****************************
;  by Jeff Frohwein
;
;  Compiled with RGBASM V1.07.
;
;  Last edit 28-Jun-97

; All include libraries are available from the 'libs' web page.

        INCLUDE "hardware.inc"
        INCLUDE "ibmpc1.inc"

lcd_WaitVRAM: MACRO
        ldh     a,[rSTAT]       ; <---+
        and     STATF_BUSY      ;     |
        jr      nz,@-4          ; ----+
        ENDM

;Version 1.2 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.2

GoRom   EQU     $c000

        SECTION "Low Ram",BSS

ExecStore       DS      16              ; Storage for ExecStart routine
_PadData        DS      1
_PadDataEdge    DS      1
buttonPressed   DS      1
autoRunTimeout  DS      2

;High RAM Assignments

SPR_DMA equ $ff80

; NOTE: IF YOU ACCESS THESE VARIABLES YOU MUST
; USE THE 'LDH' COMMAND.


        SECTION "Org $00",HOME
        ret

        SECTION "Org $40",HOME[$40]     ; Vertical Blank Interrupt
        reti

        SECTION "Org $50",HOME[$50]     ; Timer Overflow Interrupt
        reti

        DB      "Date 970928"

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      begin

        NINTENDO_LOGO

;Rom Header Info

 db "C3 MULTICART 1.0"        ; Cart name   16bytes
 db 0,0,0                     ; Not used
 db 0                         ; Cart type   ROM Only
 db 0                         ; ROM Size    32k
 db 0                         ; RAM Size     8k
 db 1,1                       ; Maker ID
 db 0                         ; Version     =0
 db $e2                       ; Complement check (important)
 dw $c40e                     ; Checksum (not important)

        SECTION "Org $4000",HOME[$4000]

        jp begin

ROM_TOTAL               EQU     $4003
ROM_DATA_STORAGE        EQU     $4004

ROM_PARTITION_SET1      EQU     0
ROM_CODE_START1         EQU     1
ROM_TITLE_DISPLAY       EQU     3
ROM_DATA_EACH_ROM       EQU     19

        DB      3                       ;ROM total

        DB      0                       ;Partition Setting
        DW      $150                    ;ROM code start
        DB      "EMPTY           "      ;ROM title display

        DB      2
        DW      $100
        DB      "EMPTY           "

        DB      3
        DW      $100
        DB      "EMPTY           "

        DB      1
        DW      $100
        DB      "                "

begin:
        di
                  ; The stack initializes to $FFFE

        xor    a               ; a = 0
        ldh    [rIF],a         ; clear pending interrupts

        ld      a,0
        ldh    [rIE],a         ; disable interrupts

        ld      a,0
        ldh     [$26],a        ; kill all sounds

        lcd_WaitVRAM           ; Must be in VBL before turning the screen off.


        ld     a,%00010101     ; LCD Controller = Off [No picture on screen]
                               ; WindowBank = $9800 [Not used]
                               ; Window = OFF
                               ; BG Chr = $8000
                               ; BG Bank= $9800
                               ; OBJ    = 8x16
                               ; OBJ    = Off
                               ; BG     = On
        ldh     [rLCDC],a


; Load char set

        ld      bc,TileData
        call    move_char        ; Move the charset to $8000

        ld      a,$e4
        ldh     [rBGP],a

; Clear Screen

        ld      hl,$9800
        ld      bc,32*18
        ld      a,32
        call    mem_Set

        ld      hl,MakeSel
        ld      de,$9800+1+(6*32)
        ld      bc,MakeSelEnd-MakeSel
        call    mem_Copy

; Draw Rom titles

        ld      hl,ROM_DATA_STORAGE + ROM_TITLE_DISPLAY
        ld      de,$9800+3+(8*32)
        ld      a,[ROM_TOTAL]
draw_titles:
        push    af
        push    hl
        push    de
        ld      bc,16
        call    mem_Copy
        pop     de

        ld      hl,32           ; de = de + 32
        add     hl,de
        ld      d,h
        ld      e,l

        pop     hl

        ld      bc,ROM_DATA_EACH_ROM
        add     hl,bc

        pop     af
        dec     a
        jr      nz,draw_titles

        ld     a,%10010101      ; LCD Controller = On
        ldh    [rLCDC],a

; Load RAM bootstrap ROM exec routine

        ld      hl,ExecStart
        ld      de,GoRom
        ld      bc,ExecEnd-ExecStart
        call    mem_Copy

        xor     a
        ld      [buttonPressed],a       ; No button pressed yet

        ld      hl,10000                ; delay for approximately 10 secs
        ld      a,l
        ld      [autoRunTimeout],a
        ld      a,h
        ld      [autoRunTimeout+1],a

        ld      d,0

;Draw program selection pointer

draw_pointer:

        ld      e,d
        inc     e
        ld      hl,$9800+1+(7*32)
        ld      bc,32
draw_p:
        add     hl,bc
        dec     e
        jr      nz,draw_p

; Erase old pointer.... On whatever line it's on.

        lcd_WaitVRAM
        ld      a,32
        ld      [$9800+1+(8*32)],a
        lcd_WaitVRAM
        ld      a,32
        ld      [$9800+1+(9*32)],a
        lcd_WaitVRAM
        ld      a,32
        ld      [$9800+1+(10*32)],a
        lcd_WaitVRAM
        ld      a,32
        ld      [$9800+1+(11*32)],a

; Draw new selection pointer at this position

        lcd_WaitVRAM
        ld      a,16
        ld      [hl],a

; Delay between draws & keypad read

        ld      hl,$8000
delay:  dec     hl
        ld      a,h
        or      l
        jr      nz,delay

; Check for auto-run
AutoRunTest:
        ld      a,220                       ; small delay
small_delay:
        dec     a
        jr      nz,small_delay

        ld      a,[buttonPressed]
        or      a                       ; has a button been pressed yet?
        jr      nz,TestUp               ; yes

        ld      a,[autoRunTimeout]      ; decrement timeout counter
        ld      l,a
        ld      a,[autoRunTimeout+1]
        ld      h,a

        dec     hl

        ld      a,l
        ld      [autoRunTimeout],a
        ld      a,h
        ld      [autoRunTimeout+1],a

        ld      a,h
        or      l                       ; Has timeout clock reached zero ?
        jr      nz,TestUp               ; no, user input hasn't timed out yet

        jp      Prog4                   ; Auto run first ROM image in list

; Get User input if available

TestUp:
        call    pad_Read
        bit     PADB_UP,a               ; Has user pressed up ?
        jr      z,TestDown              ; no

        ld      a,1
        ld      [buttonPressed],a       ; kill auto run since button pressed

        dec     d
        ld      a,d
        cp      $80
        jr      c,draw_pointer

        ld      a,[ROM_TOTAL]
        dec     a
        ld      d,a
        jp      draw_pointer

TestDown:
        bit     PADB_DOWN,a             ; Has user pressed down ?
        jr      z,TestStart             ; no

        ld      a,1
        ld      [buttonPressed],a       ; kill auto run since button pressed

        inc     d
        ld      a,[ROM_TOTAL]
        ld      e,a
        ld      a,d
        cp      e
        jp      c,draw_pointer

        ld      d,0
        jp      draw_pointer

TestStart:
        bit     PADB_START,a            ; Has user pressed start ?
        jr      z,AutoRunTest           ; no

; Execute program pointed to by D.

Prog4:
        ld      hl,ROM_DATA_STORAGE
        ld      a,d
Prog5:
        or      a                       ; Are we at the right table position ?
        jr      z,Prog6                 ; yes

        ld      bc,ROM_DATA_EACH_ROM
        add     hl,bc
        dec     a
        jr      Prog5

Prog6:
        ld      a,[hl+]
        ld      b,a
        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a
        ld      a,b
        jp      GoRom           ;a = partition, hl = ROM code start, Run ROM!!!!


; This routine gets loaded into RAM and
; is the bootstrap to execute ROMS.

ExecStart:
        ld      [$4000],a       ;Set memory partition
        ld      a,1
        ld      [$6000],a       ;Enable 4MbitROM/32KbyteRAM mode
        xor     a
        ld      [$2100],a       ;Set ROM bank to 0
        jp      [hl]            ;Execute ROM
ExecEnd:

;***************************************************************************
;*
;* mem_Copy - "Copy" a memory region
;*
;* input:
;*   hl - pSource
;*   de - pDest
;*   bc - bytecount
;*
;***************************************************************************
mem_Copy:
	inc	b
	inc	c
	jr	.skip
.loop	ld	a,[hl+]
	ld	[de],a
	inc	de
.skip	dec	c
	jr	nz,.loop
	dec	b
	jr	nz,.loop
	ret

;***************************************************************************
;*
;* mem_Set - "Set" a memory region
;*
;* input:
;*    a - value
;*   hl - pMem
;*   bc - bytecount
;*
;***************************************************************************
mem_Set:
	inc	b
	inc	c
	jr	.skip
.loop	ld	[hl+],a
.skip	dec	c
	jr	nz,.loop
	dec	b
	jr	nz,.loop
	ret

move_char:
 ld     hl,$8000
 ld     de,$1000
lp1:
 ld     a,[bc]
 ld     [hl+],a
 ld     [hl+],a
 inc    bc
 dec    e
 jr     nz,lp1
 dec    d
 jr     nz,lp1
 ret

;***************************************************************************
;*
;* pad_Read - Read the joypad
;*
;* output:
;*   _PadData & A     - joypad matrix
;*   _PadDataEdge & B - edge data: which buttons were pressed since last time
;*                       this routine was called
;*
;***************************************************************************
pad_Read:
        ld      a,P1F_5
        ld      [_HW+rP1],a        ;turn on P15

        ld      a,[_HW+rP1]        ;delay
        ld      a,[_HW+rP1]
	cpl
        and     $0f
	swap	a
	ld	b,a

        ld      a,P1F_4
        ld      [_HW+rP1],a     ;turn on P14
        ld      a,[_HW+rP1]     ;delay
        ld      a,[_HW+rP1]
        ld      a,[_HW+rP1]
        ld      a,[_HW+rP1]
        ld      a,[_HW+rP1]
        ld      a,[_HW+rP1]
	cpl
        and     $0f
	or	b
        ld      b,a

	ld	a,[_PadData]
	xor	a,b
	and	a,b
	ld	[_PadDataEdge],a
	ld	a,b
	ld	[_PadData],a
        push    af

	ld	a,P1F_5|P1F_4
	ld	[_HW+rP1],a

        ld      a,[_PadDataEdge]
        ld      b,a
        pop     af
	ret

TileData:
        chr_IBMPC1 1,4          ; include IBMPC char set

MakeSel:
       DB      "- Make Selection -"
MakeSelEnd:

