/*
 *  Composite multicart ROM image Generator
 *        by Jeff Frohwein
 * Last Edit: 28-Sept-97
 *
 * Compiled with Microsoft C/C++ V6.0
 *
 * V1.0 - Initial release.
 *
 */

#include <stdio.h>
#include <string.h>

/* required to keep LINT happy */
#define FPrintF (void) fprintf
#define FPutC (void) fputc
#define PrintF (void) printf
#define StrCat (void) strcat
#define StrCpy (void) strcpy

/* Global Variables */
FILE *ifp, *ifp2, *ifp3, *mfp, *ofp;
int numInputFiles, RomStart;
int SaveRam;
char Title1[17];
char Title2[17];
char Title3[17];

void exit (int status);

void Usage (void)
   {
   PrintF(" Usage: mult out[.mc1] in1[.gb] [in2[.gb]] [in3[.gb]]\n\n");
   }

void OpenFiles (int argc, char **argv, int arg)
   {
   int c, i;
   char fname[20];

   /* Clear cart titles */
   for (i=0; i<17; i++)
      {
      Title1[i] = 0;
      Title2[i] = 0;
      Title3[i] = 0;
      }

   StrCpy(fname, "menu.gb");

   /* Open menu code */
   if ((mfp = fopen(fname, "rb")) == NULL)
      {
      PrintF("Error opening menu file '%s'.\n", fname);
      exit(1);
      }

   if (argc-arg>1)
      /* Get output filename */
      StrCpy(fname, argv[arg+1]);
   else
      /* If no name, exit */
      {
      Usage();
      exit(1);
      }

   /* Add default extension to filename if left off */
   if (strchr(fname, '.') == NULL)
      StrCat(fname, ".mc1");

   if ((ofp = fopen(fname, "wb")) == NULL)
      {
      PrintF("Error opening output file '%s'.\n", fname);
      (void) fclose(mfp);
      exit(1);
      }
   else
      PrintF("Writing data to '%s'.\n", fname);

   if (argc-arg>2)
      /*Get input filename #1*/
      StrCpy(fname, argv[arg+2]);
   else
      /*No input name, exit */
      {
      PrintF("Error - No input file given.\n");
      (void) fclose(mfp);
      (void) fclose(ofp);
      exit(1);
      }

   /* Add default extension to filename if left off */
   if (strchr(fname, '.') == NULL)
      StrCat(fname, ".gb");

   /* If file not found, print error & exit */
   if ((ifp = fopen(fname, "rb")) == NULL)
      {
      PrintF("Error opening input file '%s'.\n", fname);
      (void) fclose(mfp);
      (void) fclose(ofp);
      exit(1);
      }

   /* Check to see that input file isn't too large */
   for (i=0; i<0x149; i++)
      c = fgetc(ifp);
   if (c > 2)
      {
      PrintF("Error - ROM Size byte of input file '%s' is too large.\n", fname);
      (void) fclose(mfp);
      (void) fclose(ofp);
      exit(1);
      }

   c = fgetc(ifp);
   if (c > 2)
      {
      PrintF("Error - RAM Size byte of input file '%s' is too large.\n", fname);
      (void) fclose(mfp);
      (void) fclose(ofp);
      exit(1);
      }

   numInputFiles = 1;

   if (argc-arg>3)
      {
      /*Get input filename #2*/
      numInputFiles++;
      StrCpy(fname, argv[arg+3]);

      /* Add default extension to filename if left off */
      if (strchr(fname, '.') == NULL)
         StrCat(fname, ".gb");

      /* If file not found, print error & exit */
      if ((ifp2 = fopen(fname, "rb")) == NULL)
         {
         PrintF("Error opening input file '%s'.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         exit(1);
         }

      /* Skip over header info */
      for (i=0; i<0x134; i++)
         (void) fgetc(ifp2);

      /* Get title name from cart */
      for (i=0; i<16; i++)
         Title2[i] = fgetc(ifp2);
      Title2[16] = 0;

      /* Skip over 144 to 148 */
      for (i=0; i<5; i++)
         c = fgetc(ifp2);

      /* Check to see that input file isn't too large */
      if (c > 2)
         {
         PrintF("Error - ROM Size byte of input file '%s' is too large.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         exit(1);
         }

      c = fgetc(ifp2);
      if (c > 2)
         {
         PrintF("Error - RAM Size byte of input file '%s' is too large.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         exit(1);
         }

      /* If ROM #2 uses cart RAM reserve TWO banks for it */
      if (c != 0)
         SaveRam = 1;
      else
         SaveRam = 0;
      }

   if (argc-arg>4)
      {
      /*Get input filename #3*/
      numInputFiles++;
      StrCpy(fname, argv[arg+4]);

      /* Add default extension to filename if left off */
      if (strchr(fname, '.') == NULL)
         StrCat(fname, ".gb");

      /* If file not found, print error & exit */
      if ((ifp3 = fopen(fname, "rb")) == NULL)
         {
         PrintF("Error opening input file '%s'.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         (void) fclose(ifp2);
         exit(1);
         }

      /* Skip over header info */
      for (i=0; i<0x134; i++)
         (void) fgetc(ifp3);

      /* Get title name from cart */
      for (i=0; i<16; i++)
         Title3[i] = fgetc(ifp3);
      Title3[16] = 0;

      /* Skip over 144 to 148 */
      for (i=0; i<5; i++)
         c = fgetc(ifp3);

      /* Check to see that input file isn't too large */
      if (c > 2)
         {
         PrintF("Error - ROM Size byte of input file '%s' is too large.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         (void) fclose(ifp2);
         exit(1);
         }

      c = fgetc(ifp3);
      if (c > 2)
         {
         PrintF("Error - RAM Size byte of input file '%s' is too large.\n", fname);
         (void) fclose(mfp);
         (void) fclose(ofp);
         (void) fclose(ifp);
         (void) fclose(ifp2);
         exit(1);
         }
      }
   }

void CloseFiles (void)
   {
   fclose(ifp);
   fclose(ifp2);
   fclose(ifp3);
   fclose(ofp);
   fclose(mfp);

   PrintF("Done.\n");
   }

void ErrorExit (void)
   {
   fclose(ifp);
   fclose(ifp2);
   fclose(ifp3);
   fclose(ofp);
   fclose(mfp);
   exit(1);
   }

void ProcessFiles (void)
   {
   int c,i;
   long cnt;

   rewind(ifp);

   /*  Copy 00 to 101 */
   cnt = 0;
   while ( (!feof(ifp)) &&
           (cnt < 0x102) )
      {
      c = fgetc(ifp);
      FPutC(c, ofp);
      cnt++;
      }

   /* Save ROM #1 jump address */
   c = fgetc(ifp);
   RomStart = (fgetc(ifp) * 256) + c;

   /* Patch 102 & 103 with jump to bank switch code */
   FPutC(0x34, ofp);
   FPutC(0x01, ofp);

   /*  Copy 104 to 133 */
   cnt = 0;
   while ( (!feof(ifp)) &&
           (cnt < 0x30) )
      {
      c = fgetc(ifp);
      FPutC(c, ofp);
      cnt++;
      }

   /* Read cart title */
   for (i=0; i<16; i++)
      Title1[i] = fgetc(ifp);
   Title1[16] = 0;

   PrintF("Installing '%s'\n", Title1);

   /* Install bank switch code*/
   FPutC(0x3e, ofp);                    // ld a,8
   FPutC(0x08, ofp);
   FPutC(0xea, ofp);                    // ld (2100),a
   FPutC(0x00, ofp);
   FPutC(0x21, ofp);
   FPutC(0xc3, ofp);                    // jp 4000
   FPutC(0x00, ofp);
   FPutC(0x40, ofp);
   for (i=0; i<8; i++)                  // add 8 nops
      FPutC(0x00, ofp);

   /*  Copy 144 to 148 */
   cnt = 0;
   while ( (!feof(ifp)) &&
           (cnt < 4) )
      {
      c = fgetc(ifp);
      FPutC(c, ofp);
      cnt++;
      }

   c = fgetc(ifp);
   /* Install correct ROM size */
   if (numInputFiles > 1)
      FPutC(0x04, ofp);
   else
      FPutC(0x03, ofp);

   /*  Copy rest of rom */
   cnt = 0x149;
   while ( cnt < 131072)
      {
      if (!feof(ifp))
         c = fgetc(ifp);
      else
         c = 0xff;
      FPutC(c, ofp);
      cnt++;
      }

    /* Ignore first 4000h bytes of menu file */
    cnt = 0;
    while ( (!feof(mfp)) &&
            (cnt < 0x4000) )
       {
       (void) fgetc(mfp);
       cnt++;
       }

    if (feof(mfp))
       {
       PrintF("Error - Menu program is too small!\n");
       }

   /*  Copy menu jp command (4000 - 4002 */
   cnt = 0;
   while ( (!feof(mfp)) &&
           (cnt < 3) )
      {
      c = fgetc(mfp);
      FPutC(c, ofp);
      cnt++;
      }

    /* Ignore ROM cart info */
    cnt = 0;
    while ( (!feof(mfp)) &&
            (cnt < (19*4)+1) )
       {
       (void) fgetc(mfp);
       cnt++;
       }

   /* Install correct ROM total */
   FPutC(numInputFiles+SaveRam, ofp);

   /* Install ROM #1 info*/
   FPutC(0, ofp);                       /* Partition setting 0 */
   FPutC((RomStart & 0xff), ofp);       /* Install entry point */
   FPutC((RomStart >> 8), ofp);

   for (i=0; i<16; i++)                 /* Install cart title */
      if (Title1[i] == 0)
         FPutC(0x20, ofp);
      else
         FPutC(Title1[i], ofp);

   /* Install ROM #2 (save ram #1) info*/
   FPutC(2, ofp);                       /* Partition setting 2 */
   FPutC(0x00, ofp);                    /* Install entry point */
   FPutC(0x01, ofp);

   for (i=0; i<16; i++)                 /* Install cart title */
      if (Title2[i] == 0)
         FPutC(0x20, ofp);
      else
         FPutC(Title2[i], ofp);

   if (SaveRam == 1)
      {
      /* Install ROM #2 (save ram #2) info*/
      FPutC(3, ofp);                       /* Partition setting 2 */
      FPutC(0x00, ofp);                    /* Install entry point */
      FPutC(0x01, ofp);

      for (i=0; i<16; i++)                 /* Install cart title */
         if (Title2[i] == 0)
            FPutC(0x20, ofp);
         else
            FPutC(Title2[i], ofp);
      }

   /* Install ROM #3 info*/
   FPutC(1, ofp);                       /* Partition setting 2 */
   FPutC(0x00, ofp);                    /* Install entry point */
   FPutC(0x01, ofp);

   for (i=0; i<16; i++)                 /* Install cart title */
      if (Title3[i] == 0)
         FPutC(0x20, ofp);
      else
         FPutC(Title3[i], ofp);

   if (SaveRam == 0)
      {
      /* Install no info in last block*/
      FPutC(0, ofp);                       /* Partition setting null */
      FPutC(0x00, ofp);                    /* Install null entry point */
      FPutC(0x00, ofp);

      for (i=0; i<16; i++)                 /* Install cart title */
         FPutC(0x20, ofp);
      }

   /*  Copy rest of menu rom */
   cnt = (19*4)+4;
   while ( cnt < 131072)
      {
      if (!feof(mfp))
         c = fgetc(mfp);
      else
         c = 0xff;
      FPutC(c, ofp);
      cnt++;
      }

   if (numInputFiles > 1)
      {
      /* Install ROM #2 */
      PrintF("Installing '%s'\n", Title2);

      rewind(ifp2);

      /*  Copy rom #2, fill unused space with FF */
      cnt = 0;
      while ( cnt < 131072)
         {
         if (!feof(ifp2))
            c = fgetc(ifp2);
         else
            c = 0xff;
         FPutC(c, ofp);
         cnt++;
         }
      }

   if (numInputFiles > 2)
      {
      /* Install ROM #3 */
      PrintF("Installing '%s'\n", Title3);

      rewind(ifp3);

      /*  Copy rom #3, fill unused space with FF */
      cnt = 0;
      while ( cnt < 131072)
         {
         if (!feof(ifp3))
            c = fgetc(ifp3);
         else
            c = 0xff;
         FPutC(c, ofp);
         cnt++;
         }
      }
   else
      if (numInputFiles > 1)
         {
         /*  Fill rom #3 unused space with FF */
         cnt = 0;
         while ( cnt < 131072)
            {
            FPutC(0xff, ofp);
            cnt++;
            }
         }
   }

void main(int argc, char **argv)
   {
   int arg;
   int argp = 0;

   PrintF("Composite multicart ROM image Generator V1.0, Jeff Frohwein\n\n");

   for (arg = 1; arg < argc; arg++)
      {

      /* process command args */
      if (argv[arg][0] == '-')
         {
         argp++;
         switch (argv[arg][1])
            {
         case 'c':
         case 'C':
//            Colors = TWO_COLOR;
            break;
         case 'r':
         case 'R':
//            Style = RGBDS;
            break;
         case 't':
         case 'T':
//            Style = TASM;
            break;
         default:
            Usage();
            exit(1);
            }
         }
      }

//  PrintF("style=%d\n", Style);

   OpenFiles(argc, argv, argp);
//   PrintF ("argc,argp = %d,%d\n", argc, argp);

   ProcessFiles();

   CloseFiles();
   exit(0);
   }
